START TRANSACTION;

CREATE TABLE `auth_tokens` (
  `Token` char(40) charset ascii NOT NULL,
  `AccountID` int(11) NOT NULL,
  `Created` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `Accessed` TIMESTAMP NOT NULL DEFAULT 0,
  `Mask` int(11) NOT NULL DEFAULT 0,
  UNIQUE KEY `Token` (`Token`),
  KEY `AccountID` (`AccountID`)
);

ALTER TABLE `auth_tokens`
  ADD CONSTRAINT `auth_tokens_ibfk_1` FOREIGN KEY (`AccountID`) REFERENCES `accounts` (`ID`);


DELIMITER //

-- returns:
-- -1 : invalid token
-- >0 : accountID
DROP FUNCTION IF EXISTS SignInByToken//
CREATE FUNCTION SignInByToken (pToken char(40) charset ascii) RETURNS int(11) # account id
NO SQL
BEGIN
    DECLARE vAccountID INT DEFAULT -1;

    SELECT auth_tokens.AccountID INTO vAccountID
        FROM auth_tokens
        WHERE auth_tokens.Token = pToken
        LIMIT 1;

    IF vAccountID = -1 THEN
        RETURN -1; -- token not found
    END IF;

    UPDATE auth_tokens
        SET Accessed = CURRENT_TIMESTAMP
        WHERE auth_tokens.Token = pToken
        LIMIT 1;

    INSERT INTO logs_signin(AccountID) VALUES (vAccountID);
    RETURN vAccountID;
END //

DROP FUNCTION IF EXISTS DeleteAuthToken//
CREATE FUNCTION DeleteAuthToken(pToken char(40) CHARSET ascii) RETURNS int(11)
NO SQL
BEGIN
    DELETE FROM auth_tokens
        WHERE auth_tokens.Token = pToken;
    RETURN ROW_COUNT()-1;
END//

DROP FUNCTION IF EXISTS CreateAuthToken//
CREATE FUNCTION CreateAuthToken(
    pAccountID int(11),
    pRightMask int(11)
) RETURNS char(40) charset ascii
NO SQL
BEGIN
    DECLARE vCounter int(11) DEFAULT 666;
    DECLARE vCreated boolean DEFAULT FALSE;
    DECLARE vSeed int(11) DEFAULT FLOOR(4294967296 * RAND(CURRENT_TIMESTAMP ^ LAST_INSERT_ID() ^ (pAccountID << 10)));
    DECLARE vToken char(40) charset ascii DEFAULT SHA1(vSeed);

    WHILE NOT vCreated AND vCounter > 0 DO BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION SET vCreated = FALSE;
        SET vToken = SHA1(RAND());
        INSERT INTO auth_tokens(Token, AccountID, Mask) VALUES (vToken, pAccountID, pRightMask);

        SET vCreated = TRUE;
        SET vCounter = vCounter - 1;
    END; END WHILE;

    RETURN IF(vCreated, vToken, '');
END//

-- pAccountID == 0 - get tokens of all users
DROP PROCEDURE IF EXISTS GetAuthTokens//
CREATE PROCEDURE GetAuthTokens(
    IN pAccountID int(11))
BEGIN
    -- accountID, Created, Accessed, Mask
    IF pAccountID = 0 THEN
        SELECT auth_tokens.*
            FROM auth_tokens;
    ELSE
        SELECT auth_tokens.*
            FROM auth_tokens
            WHERE auth_tokens.AccountID = pAccountID;
    END IF;
END//




DELIMITER ;

COMMIT ;
